"use strict";
exports.__esModule = true;
exports.EventBus = exports.registerEventListener = exports.removeEvent = exports.fireEvent = void 0;
// Object holding the all the handlers for events mapped with the event names
var events = {};
/**
 * Call all the registered handlers for the given event name with the given arguments
 *
 * @param {string} name of the event
 * @param {any[]} arguments to be passed to the handlers
 */
function fireEvent(name) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    // If no such event is registered, then no need to call
    if (!events[name])
        return;
    var eventListeners = events[name];
    // Calling each of the listeners in the order of their registration
    eventListeners.forEach(function (listener) {
        listener.apply(void 0, args);
    });
}
exports.fireEvent = fireEvent;
/**
 * Removes a given event from events registered
 *
 * @param {string} name of the event to remove
 */
function removeEvent(eventName) {
    delete events[eventName];
}
exports.removeEvent = removeEvent;
/**
 * Registers the given listener for the given event
 *
 * @param {string} Name of event
 * @param {Function} listener to be registered
 * @param {boolean} if parameter once set to `true` calls this listener only the first time the given event is fired
 */
function registerEventListener(eventName, listener, once) {
    if (once === void 0) { once = false; }
    if (!events[eventName])
        events[eventName] = [];
    var eventListeners = events[eventName];
    // if the listener not intended to be run only once, directly add the listener and return immediately
    if (!once) {
        eventListeners.push(listener);
        return;
    }
    // Since, listener would be added to the end, hence the index would be equal to the current number of listeners
    var indexOfListener = eventListeners.length;
    /**
     * Function which itself calls the given listener function, and then de-registers itself as one of the listeners for the given event
     */
    var oneTimeListener = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        listener.apply(void 0, args);
        // Remove this listener
        eventListeners.splice(indexOfListener, 1);
    };
    // Register one time listener for the given event
    eventListeners.push(oneTimeListener);
}
exports.registerEventListener = registerEventListener;
exports.EventBus = {
    $emit: fireEvent,
    $off: removeEvent,
    $on: registerEventListener
};
//# sourceMappingURL=eventBus.js.map